/*	main.cpp	*/

/*	This simple program loads a sound file in raw CD format and plays	*/
/*	it as a (mono) sound effect - your choice between 44 kHz 16 bit	*/
/*	and 11 kHz 8 bit format (simulated)								*/

#include <Application.h>
#include <Window.h>
#include <Button.h>
#include <stdio.h>

#include "be-sfx.h"


#define	PLAY_A	'plya'
#define	PLAY_B	'plyb'

#define	SET_PAN	'span'

class PanControl :
	public BView
{
		float					pos;
public:
								PanControl(
									BRect			area) :
									BView(area, "pan", B_FOLLOW_NONE, B_WILL_DRAW)
									{
										pos = area.Width()/2.0;
									}
		void					Draw(
									BRect			area)
									{
										const char * strl = "Left";
										const char * strr = "Right";
										BRect b = Bounds();
										SetHighColor(210, 210, 210, 0);
										FillRect(b, B_SOLID_HIGH);
										SetHighColor(128,128,128,0);
										SetFontSize(18);
										DrawString(strl, BPoint(4, b.bottom-4));
										DrawString(strr, BPoint(b.right-StringWidth(strr)-4, b.bottom-4));
										StrokeLine(BPoint(b.Width()/2.0, b.top),
											BPoint(b.Width()/2.0, b.bottom));
										SetHighColor(0,0,0,0);
										StrokeLine(BPoint(pos, b.top), 
											BPoint(pos, b.bottom));
									}
		void					MouseDown(
									BPoint			where)
									{
										uint32 buttons;
										float old = pos;
										float orig = pos;
										BRect bnd = Bounds();
										do {
											pos = where.x;
											if (pos < bnd.left)
												pos = bnd.left;
											if (pos > bnd.right)
												pos = bnd.right;
											if (pos != old)
											{
												Draw(Bounds());
												Flush();
												old = pos;
											}
											else
												snooze(20000);
											GetMouse(&where, &buttons);
										} while (buttons != 0);
										if (pos != orig)
										{
											BMessage msg(SET_PAN);
											msg.AddInt32("pan", GetPan());
											Window()->PostMessage(&msg);
										}
									}
		int						GetPan()
									{
										float h = Bounds().Width();
										int p = 127.0*(pos*2-h)/h;
										return p;
									}
};


class SFXWindow :
	public BWindow
{
public:
								SFXWindow(
									void *			data,
									int				size);
								~SFXWindow();

		void					MessageReceived(
									BMessage *		message);

		SoundEffect *			a;
		SoundEffect8_11 *		b;
		EffectsPlayer *			fx;
		PanControl *			panner;
		effect_id				playingEffect;
};


SFXWindow::SFXWindow(
	void *				data,
	int					size) :
	BWindow(
		BRect(200,200,474,270),
		"Sound Effects",
		B_TITLED_WINDOW,
		B_NOT_CLOSABLE | B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
	playingEffect = -1;
	short * data1 = (short *)malloc(size/2);	/*	stereo->mono	*/
	char * data2 = (char *)malloc(size/16);
	/*	create the mixed mono 16/44 and the downsampled mono 8/11	*/
	for (int ix=0; ix<size/4; ix++)
	{
		/*	mix stereo to mono	*/
		/*	sample file is in big-endian format	*/
#if B_HOST_IS_LENDIAN
		short v = (B_SWAP_INT16(((short *)data)[ix*2])+B_SWAP_INT16(((short *)data)[ix*2+1]))/2;
#else
		short v = (((short *)data)[ix*2]+((short *)data)[ix*2+1])/2;
#endif
		data1[ix] = v;
		if (!(ix&3))
			data2[ix/4] = v>>8;
	}
	free(data);
	a = new SoundEffect(false, data1, size/4);	/*	bytes -> frames	*/
	/*	loop the low-quality sound 2 times (so it plays 3 times) to test looping	*/
	b = new SoundEffect8_11(data2, size/16, 2);	/*	by->fr, st->mo, 44->11, 16->8	*/

	BButton *btn = new BButton(BRect(10,10,95,30), "44", "44 kHz 16 bit",
		new BMessage(PLAY_A));
	AddChild(btn);
	btn = new BButton(BRect(105,10,190,30), "11", "11 kHz 8 bit", 
		new BMessage(PLAY_B));
	AddChild(btn);

	panner = new PanControl(BRect(10,40,264,60));
	AddChild(panner);

	fx = new EffectsPlayer;
	if (fx->InitCheck())
	{
		fprintf(stderr, "InitCheck() failed: %08x\n", fx->InitCheck());
		be_app->PostMessage(B_QUIT_REQUESTED);
	}
}


SFXWindow::~SFXWindow()
{
	delete fx;
	delete a;
	delete b;
}


void
SFXWindow::MessageReceived(
	BMessage *			message)
{
	switch (message->what)
	{
	case PLAY_A:
		playingEffect = fx->StartEffect(a, 127, panner->GetPan());
		break;
	case PLAY_B:
		playingEffect = fx->StartEffect(b, 127, panner->GetPan());
		break;
	case SET_PAN:
		{
			int32 i = 0;
			message->FindInt32("pan", &i);
			fx->SetEffectVolPan(playingEffect, -1, i);
		}
		break;
	default:
		BWindow::MessageReceived(message);
		break;
	}
}


void
main(
	int argc,
	char *argv[])
{
	if (argc != 2)
	{
		fprintf(stderr, "usage: soundfx <filename>\n");
		exit(1);
	}
	FILE *f = fopen(argv[1], "r");
	if (!f)
	{
		fprintf(stderr, "cannot open %s\n", argv[1]);
		exit(1);
	}
	fseek(f, 0, 2);
	int size = ftell(f);	/*	assume stereo, 44, 16	*/
	fseek(f, 0, 0);
	void * data = malloc(size);
	fread(data, 1, size, f);
	fclose(f);

	BApplication app("application/x-hplus-sound-fx");
	SFXWindow *w = new SFXWindow(data, size);
	w->Show();
	app.Run();
}
